
// misc commands that don't apply to the PI chipset but are define PI_ by the 
// motion controller and included here for symmetry

// define PI_different types for the different command
// this allows the appropriate function to be called
//
// Each command sends and/or receives set amounts of data
// so define primatives for the main types
// CMD_WRITE_READ1 / CMD_WRITE_READ2 are currently implemented
// using CMD_WRITE1 and BLANK_READs - matt 

#ifndef __PI_CMD__
#define __PI_CMD__
typedef int             PI_CMD_ONLY;
typedef int             PI_CMD_READ1;
typedef short           PI_CMD_READ2;
typedef int             PI_CMD_WRITE1;
typedef int             PI_CMD_WRITE2;
typedef int             PI_CMD_WRITE3;
typedef int             PI_CMD_WRITE_READ1;
typedef short           PI_CMD_WRITE_READ2;
typedef int  PI_DATA;
typedef unsigned short  PI_CMD;

// send a command with no data IO
#define PI_CLEAR_INTERRUPT		(PI_CMD_ONLY)0xac
#define PI_NO_OPERATION			(PI_CMD_ONLY)0x00
#define PI_RESET			(PI_CMD_ONLY)0x39
#define PI_UPDATE			(PI_CMD_ONLY)0x1a

// commands that get something but require an argument
#define PI_READ_ANALOG			(PI_CMD_WRITE_READ1)0xef
#define PI_READ_BUFFER			(PI_CMD_WRITE_READ2)0xc9
#define PI_READ_IO			(PI_CMD_WRITE_READ1)0x83
#define PI_GET_TRACE_VARIABLE		(PI_CMD_WRITE_READ1)0xb7
#define PI_GET_BREAKPOINT		(PI_CMD_WRITE_READ1)0xd5
#define PI_GET_BREAKPOINT_VALUE		(PI_CMD_WRITE_READ2)0xd7
#define PI_GET_BUFFER_FUNCTION		(PI_CMD_WRITE_READ1)0xcb
#define PI_GET_BUFFER_LENGTH		(PI_CMD_WRITE_READ2)0xc3
#define PI_GET_BUFFER_READ_INDEX	(PI_CMD_WRITE_READ2)0xc7
#define PI_GET_BUFFER_START		(PI_CMD_WRITE_READ2)0xc1
#define PI_GET_BUFFER_WRITE_INDEX	(PI_CMD_WRITE_READ2)0xc5

// commands that get something
#define PI_GET_ACCELERATION		(PI_CMD_READ2)0x4c
#define PI_GET_ACTIVITY_STATUS		(PI_CMD_READ1)0xa6
#define PI_GET_ACTUAL_POSITION		(PI_CMD_READ2)0x37
#define PI_GET_ACTUAL_VELOCITY		(PI_CMD_READ2)0xad
#define PI_GET_AUTO_STOP_MODE		(PI_CMD_READ1)0xd3
#define PI_GET_AXIS_MODE		(PI_CMD_READ1)0x88
#define PI_GET_AXIS_OUT_SOURCE		(PI_CMD_READ1)0xee
#define PI_GET_CAPTURE_SOURCE		(PI_CMD_READ2)0xd9
#define PI_GET_CAPTURE_VALUE		(PI_CMD_READ2)0x36
#define PI_GET_CHECKSUM			(PI_CMD_READ2)0xf8
#define PI_GET_COMMANDED_ACCELERATION	(PI_CMD_READ2)0xa7
#define PI_GET_COMMANDED_POSITION	(PI_CMD_READ2)0x1d
#define PI_GET_COMMANDED_VELOCITY	(PI_CMD_READ2)0x1e
#define PI_GET_CURRENT_MOTOR_COMMAND	(PI_CMD_READ1)0x3a
#define PI_GET_DECELERATION		(PI_CMD_READ2)0x92
#define PI_GET_DERIVATIVE		(PI_CMD_READ1)0x9b
#define PI_GET_DERIVATIVE_TIME		(PI_CMD_READ1)0x9d
#define PI_GET_DIAGNOSTIC_PORT_MODE	(PI_CMD_READ1)0x8a
#define PI_GET_ENCODER_MODULUS		(PI_CMD_READ1)0x8d
#define PI_GET_ENCODER_SOURCE		(PI_CMD_READ1)0xdb
#define PI_GET_EVENT_STATUS		(PI_CMD_READ1)0x31
#define PI_GET_GEAR_MASTER		(PI_CMD_READ1)0xaf
#define PI_GET_GEAR_RATIO		(PI_CMD_READ2)0x59
#define PI_GET_HOST_IO_ERROR		(PI_CMD_READ1)0xa5
#define PI_GET_INTEGRAL			(PI_CMD_READ2)0x9a
#define PI_GET_INTEGRATION_LIMIT	(PI_CMD_READ2)0x96
#define PI_GET_INTERRUPT_AXIS		(PI_CMD_READ1)0xe1
#define PI_GET_INTERRUPT_MASK		(PI_CMD_READ1)0x56
#define PI_GET_JERK			(PI_CMD_READ2)0x58
#define PI_GET_KAFF			(PI_CMD_READ1)0x94
#define PI_GET_KD			(PI_CMD_READ1)0x52
#define PI_GET_KI			(PI_CMD_READ1)0x51
#define PI_GET_KOUT			(PI_CMD_READ1)0x9f
#define PI_GET_KP			(PI_CMD_READ1)0x50
#define PI_GET_KVFF			(PI_CMD_READ1)0x54
#define PI_GET_LIMIT_SWITCH_MODE	(PI_CMD_READ1)0x81
#define PI_GET_MOTION_COMPLETE_MODE	(PI_CMD_READ1)0xec
#define PI_GET_MOTOR_BIAS		(PI_CMD_READ1)0x2d
#define PI_GET_MOTOR_COMMAND		(PI_CMD_READ1)0x69
#define PI_GET_MOTOR_LIMIT		(PI_CMD_READ1)0x07
#define PI_GET_MOTOR_MODE		(PI_CMD_READ1)0xdd
#define PI_GET_OUTPUT_MODE		(PI_CMD_READ1)0x6e
#define PI_GET_POSITION			(PI_CMD_READ2)0x4a
#define PI_GET_POSITION_ERROR		(PI_CMD_READ2)0x99
#define PI_GET_POSITION_ERROR_LIMIT	(PI_CMD_READ2)0x98
#define PI_GET_PROFILE_MODE		(PI_CMD_READ2)0xa1
#define PI_GET_SAMPLE_TIME		(PI_CMD_READ1)0x61
#define PI_GET_SERIAL_PORT_MODE		(PI_CMD_READ1)0x8c
#define PI_GET_SETTLE_TIME		(PI_CMD_READ1)0xab
#define PI_GET_SETTLE_WINDOW		(PI_CMD_READ1)0xbd
#define PI_GET_SIGNAL_SENSE		(PI_CMD_READ1)0xa3
#define PI_GET_SIGNAL_STATUS		(PI_CMD_READ1)0xa4
#define PI_GET_STOP_MODE		(PI_CMD_READ1)0xd1
#define PI_GET_TIME			(PI_CMD_READ2)0x3e
#define PI_GET_TRACE_COUNT		(PI_CMD_READ2)0xbb
#define PI_GET_TRACE_MODE		(PI_CMD_READ1)0xb1
#define PI_GET_TRACE_PERIOD		(PI_CMD_READ1)0xb9
#define PI_GET_TRACE_START		(PI_CMD_READ1)0xb3
#define PI_GET_TRACE_STATUS		(PI_CMD_READ1)0xba
#define PI_GET_TRACE_STOP		(PI_CMD_READ1)0xb5
#define PI_GET_TRACKING_WINDOW		(PI_CMD_READ1)0xa9
#define PI_GET_VELOCITY			(PI_CMD_READ2)0x4b
#define PI_GET_VERSION			(PI_CMD_READ2)0x8f

// commands that set something
#define PI_ADJUST_ACTUAL_POSITION	(PI_CMD_WRITE2)0xf5
#define PI_CLEAR_POSITION_ERROR		(PI_CMD_WRITE1)0x47
#define PI_MULTI_UPDATE			(PI_CMD_WRITE1)0x5b
#define PI_RESET_EVENT_STATUS		(PI_CMD_WRITE1)0x34
#define PI_SET_ACCELERATION		(PI_CMD_WRITE2)0x90
#define PI_SET_ACTUAL_POSITION		(PI_CMD_WRITE2)0x4d
#define PI_SET_AUTO_STOP_MODE		(PI_CMD_WRITE1)0xd2
#define PI_SET_AXIS_MODE		(PI_CMD_WRITE1)0x87
#define PI_SET_AXIS_OUT_SOURCE		(PI_CMD_WRITE1)0xed
#define PI_SET_BREAKPOINT		(PI_CMD_WRITE2)0xd4  // takes two data words
#define PI_SET_BREAKPOINT_VALUE		(PI_CMD_WRITE3)0xd6
#define PI_SET_BUFFER_FUNCTION		(PI_CMD_WRITE2)0xca
#define PI_SET_BUFFER_LENGTH		(PI_CMD_WRITE3)0xc2
#define PI_SET_BUFFER_READ_INDEX	(PI_CMD_WRITE3)0xc6
#define PI_SET_BUFFER_START		(PI_CMD_WRITE3)0xc0
#define PI_SET_BUFFER_WRITE_INDEX	(PI_CMD_WRITE3)0xc4
#define PI_SET_CAPTURE_SOURCE		(PI_CMD_WRITE1)0xd8
#define PI_SET_DECELERATION		(PI_CMD_WRITE2)0x91
#define PI_SET_DERIVATIVE_TIME		(PI_CMD_WRITE1)0x9c
#define PI_SET_DIAGNOSTIC_PORT_MODE	(PI_CMD_WRITE1)0x89
#define PI_SET_ENCODER_MODULUS		(PI_CMD_WRITE1)0x8e
#define PI_SET_ENCODER_SOURCE		(PI_CMD_WRITE1)0xda
#define PI_SET_GEAR_MASTER		(PI_CMD_WRITE1)0xae
#define PI_SET_GEAR_RATIO		(PI_CMD_WRITE2)0x14
#define PI_SET_INTEGRATION_LIMIT	(PI_CMD_WRITE2)0x95
#define PI_SET_INTERRUPT_MASK		(PI_CMD_WRITE1)0x2f
#define PI_SET_JERK			(PI_CMD_WRITE2)0x13
#define PI_SET_KAFF			(PI_CMD_WRITE1)0x93
#define PI_SET_KD			(PI_CMD_WRITE1)0x27
#define PI_SET_KI			(PI_CMD_WRITE1)0x26
#define PI_SET_KOUT			(PI_CMD_WRITE1)0x9e
#define PI_SET_KP			(PI_CMD_WRITE1)0x25
#define PI_SET_KVFF			(PI_CMD_WRITE1)0x2b
#define PI_SET_LIMIT_SWITCH_MODE	(PI_CMD_WRITE1)0x80
#define PI_SET_MOTION_COMPLETE_MODE	(PI_CMD_WRITE1)0xeb
#define PI_SET_MOTOR_BIAS		(PI_CMD_WRITE1)0x0f
#define PI_SET_MOTOR_COMMAND		(PI_CMD_WRITE1)0x77
#define PI_SET_MOTOR_LIMIT		(PI_CMD_WRITE1)0x06
#define PI_SET_MOTOR_MODE		(PI_CMD_WRITE1)0xdc
#define PI_SET_OUTPUT_MODE		(PI_CMD_WRITE1)0xe0
#define PI_SET_POSITION			(PI_CMD_WRITE2)0x10
#define PI_SET_POSITION_ERROR_LIMIT	(PI_CMD_WRITE2)0x97
#define PI_SET_PROFILE_MODE		(PI_CMD_WRITE1)0xa0
#define PI_SET_SAMPLE_TIME		(PI_CMD_WRITE1)0x38
#define PI_SET_SERIAL_PORT_MODE		(PI_CMD_WRITE1)0x8b
#define PI_SET_SETTLE_TIME		(PI_CMD_WRITE1)0xaa
#define PI_SET_SETTLE_WINDOW		(PI_CMD_WRITE1)0xbc
#define PI_SET_SIGNAL_SENSE		(PI_CMD_WRITE1)0xa2
#define PI_SET_STOP_MODE		(PI_CMD_WRITE1)0xd0
#define PI_SET_TRACE_MODE		(PI_CMD_WRITE1)0xb0
#define PI_SET_TRACE_PERIOD		(PI_CMD_WRITE1)0xb8
#define PI_SET_TRACE_START		(PI_CMD_WRITE1)0xb2
#define PI_SET_TRACE_STOP		(PI_CMD_WRITE1)0xb4
#define PI_SET_TRACE_VARIABLE		(PI_CMD_WRITE2)0xb6
#define PI_SET_TRACKING_WINDOW		(PI_CMD_WRITE1)0xa8
#define PI_SET_VELOCITY			(PI_CMD_WRITE2)0x11
#define PI_WRITE_BUFFER			(PI_CMD_WRITE3)0xc8
#define PI_WRITE_IO			(PI_CMD_WRITE2)0x82

#define IO_OFFSET_IO			0
#define IO_OFFSET_MOTOR_AND_BRAKES	1

// forbidden commands on 21xx systems (pi)
/*
#define PI_GET_PHASE_COMMAND		(PI_CMD_WRITE_READ1)0xea
#define PI_INITIALIZE_PHASE		(PI_CMD_ONLY)0x7a
#define PI_GET_ACTUAL_POSITION_UNITS	(PI_CMD_READ1)0xbf
#define PI_SET_ACTUAL_POSITION_UNITS	(PI_CMD_WRITE1)0xbe
#define PI_SET_COMMUTATION_MODE		(PI_CMD_WRITE1)0xe2
#define PI_GET_COMMUTATION_MODE		(PI_CMD_READ1)0xe3
#define PI_SET_ENCODER_TO_STEP_RATIO	(PI_CMD_WRITE2)0xde
#define PI_GET_ENCODER_TO_STEP_RATIO	(PI_CMD_READ2)0xdf
#define PI_SET_NUMBER_PHASES		(PI_CMD_WRITE1)0x85
#define PI_GET_NUMBER_PHASES		(PI_CMD_READ1)0x86
#define PI_SET_PHASE_ANGLE		(PI_CMD_WRITE1)0x84
#define PI_GET_PHASE_ANGLE		(PI_CMD_READ1)0x2c
#define PI_SET_PHASE_CORRECTION_MODE	(PI_CMD_WRITE1)0xe8
#define PI_GET_PHASE_CORRECTION_MODE	(PI_CMD_READ1)0xe9
#define PI_SET_PHASE_COUNTS		(PI_CMD_WRITE1)0x75
#define PI_GET_PHASE_COUNTS		(PI_CMD_READ1)0x7d
#define PI_SET_PHASE_INITIALIZE_MODE	(PI_CMD_WRITE1)0xe4
#define PI_GET_PHASE_INITIALIZE_MODE	(PI_CMD_READ1)0xe5
#define PI_SET_PHASE_INITIALIZE_TIME	(PI_CMD_WRITE1)0x72
#define PI_GET_PHASE_INITIALIZE_TIME	(PI_CMD_READ1)0x7c
#define PI_SET_PHASE_OFFSET		(PI_CMD_WRITE1)0x76
#define PI_GET_PHASE_OFFSET		(PI_CMD_READ1)0x7b
#define PI_SET_PHASE_PRESCALE		(PI_CMD_WRITE1)0xe6
#define PI_GET_PHASE_PRESCALE		(PI_CMD_READ1)0xe7
#define PI_SET_START_VELOCITY		(PI_CMD_WRITE2)0x6a
#define PI_GET_START_VELOCITY		(PI_CMD_READ2)0x6b
#define PI_SET_STEP_RANGE		(PI_CMD_WRITE1)0xcf
#define PI_GET_STEP_RANGE		(PI_CMD_READ1)0xce
*/
#endif
