#include <pi_user.h>
#include <pi_execute.h>
#include <pi_table.h>


/*
void execute2(PI_AXIS axis, PI_E_CMD cmd, int var1, int var2 char * report){
	// this is a special case - only a few commands are handled here
	// so I will treat them specifically
	// first bomb out if this isn't a 2 variable catch
	if(PI_E_CWW & cmd){
		pi_setQMC(PI_CMD_BIT & cmd, var1, var2);
		msg=pi_table(PI_TABLE_BIT & cmd);
		if(msg==NULL)
			sprintf(report,PI_ERROR_REPORT);
		else 	sprintf(report,"%s %d (%8X) %d (%8X)",msg,var1,var1,var2,var2);
		return;
		}
	// we didn't get a catch bomb
	sprintf(report,PI_ERROR_REPORT);
	return;
	}
*/

void pi_execute(PI_AXIS axis, PI_E_CMD cmd, int var, char * report){
int result,temp;
char *msg;
	// fall through ifs
	// tidy up a possible error 
	report[0]='\0';	
	if(cmd & PI_GET_QMC){
		result=0;
		if(cmd & PI_E_CR)
			result=pi_getQMC(PI_CMD_BIT & cmd, axis);
		if(cmd & PI_E_CWR)
			result=pi_getQMC(PI_CMD_BIT & cmd, axis, var);
		msg=pi_table[PI_TABLE_BIT & cmd];
		if(msg==NULL)
			sprintf(report,PI_ERROR_REPORT);
		else	sprintf(report,"%s %d (0x%08x)",msg,result,result);
		return;
		}
	
	if(cmd & PI_SET_QMC){
		if(PI_E_CW)
			pi_setQMC(PI_CMD_BIT & cmd, axis, var);
		// next is always an error since we only passed 1 argument
		if(PI_E_CWW){
			sprintf(report,PI_ERROR_REPORT);
			return;
			}
		msg=pi_table[PI_TABLE_BIT & cmd];
		if(msg==NULL)
			sprintf(report,PI_ERROR_REPORT);
		else 	sprintf(report,"%s %d (0x%08x)",msg,var,var);
		return;
		}
	
	// library calls
	if(cmd & PI_LIB_FUN){
		switch(cmd){
			case PI_AB:
				pi_setQMC(PI_SET_STOP_MODE, axis, 1);
				pi_setQMC(PI_UPDATE, axis, 0);
				sprintf(report,"AB");
				break;

			// turns brakes on and off
			case PI_BN:
				pi_set_brake(axis,PI_BRAKE_ON);
				sprintf(report,"BH");
				break;
			case PI_BF:
				pi_set_brake(axis,PI_BRAKE_OFF);
				sprintf(report,"BF");
				break;

			// turns digital outs on and off
			case PI_CN:
				pi_DO_up(pi_n2b(var));
				sprintf(report,"CN %d",var);
				break;
			case PI_CF:
				pi_DO_down(pi_n2b(var));
				sprintf(report,"CF %d",var);
				break;

			case PI_DH:
				pi_setQMC(PI_SET_ACTUAL_POSITION,axis,0);
				sprintf(report,"DH");
				break;

			case PI_DLL:
				sprintf(report,"Linux V 0.52 July 2003");
				break;

			case PI_FEP:
				sprintf(report,"FEP - unsupported (use pi_find_home)");
				break;
			case PI_FEN:
				sprintf(report,"FEN - unsupported (use pi_find_home)");
				break;

			case PI_GH:
				pi_setQMC(PI_SET_POSITION,axis,0);
				pi_setQMC(PI_UPDATE,axis);
				sprintf(report,"GH");
				break;

			case PI_MR:
				pi_move_rel(axis,var);
				sprintf(report,"MR %d",var);
				break;
			case PI_MA:
				pi_move_abs(axis,var);
				sprintf(report,"MA %d",var);
				break;

			case PI_REC:
				// I think this is just "move off limits..."
				sprintf(report,"REC - not support yet");
				break;
			case PI_RET:
				pi_setQMC(PI_RESET_EVENT_STATUS,axis);

				// make sure we don't move
				temp=pi_getQMC(PI_GET_POSITION,axis);
				pi_setQMC(PI_SET_POSITION,axis,temp);
				// the following code looks flawed to me
				// it is copied from the DLL and I suspect the
				// code is bug prone there.
				// The problem is that there are many ways of
				// setting the velocity and in the DLL only one
				// of these keeps a record of the velocity and
				// this is not the one used throughout the library
				// hence the value returned by the DLL is not always
				// correct.  This is a problem with the PI code
				// that I have tried to emulate here.
				//
				// The reson for it is that the velocity is set
				// (by the PMD chipset) to zero following an
				// event status error.  Thus you need to reset
				// it to start moving again, the DLL tries to 
				// remember the value but may not (see above)
				// 
				// my recommendation is to remember the velocity
				// yourself and reset it after reseting the ESR
				temp=pi_getQMC(PI_GET_VELOCITY,axis);
				pi_setQMC(PI_SET_VELOCITY,axis,temp);
				
				// enable the motors
				pi_setQMC(PI_SET_MOTOR_MODE,axis,1);
				// call update
				pi_setQMC(PI_UPDATE,axis,0);				

				// here is a concat of the original code...
                        	// setQMC(RESET_EVENT_STATUS, ax, 0);
                        	// gets and sets the velocity and position
                        	// setQMC(SET_MOTOR_MODE, ax, 1);
                        	// setQMC(UPDATE, ax, 0);
				//
				sprintf(report,"RET");
				break;

			case PI_ST:
				// "stop with programmed acceleration"
				// not sure what this is supposed to do but the
				// code is essentially identical to the DLL
				pi_setQMC(PI_SET_STOP_MODE,axis,2);
				pi_setQMC(PI_UPDATE,axis);
				sprintf(report,"ST");
				break;

			case PI_TC:
				// returns the value of the digital in in hex
				// please tel me if I got the format correct!
				temp=pi_get_DI();
				sprintf(report,"TC %x",temp);
				break;
			case PI_TE:
				// returns the position error 
				// remember this is a little inaccurate if moving!
				// follows the PI example
				temp=pi_getQMC(PI_GET_ACTUAL_POSITION,axis);
				temp=temp-pi_getQMC(PI_GET_POSITION,axis);
				sprintf(report,"TE %d (%#010x)",temp,temp);
				break;

			case PI_WA:
				// time "absolute" waits hmmm
				usleep(1000*var);
				sprintf(report,"WA");// do I need to echo var here?
			break;
			case PI_WC:
				// wait for an a2d channel to go high
				temp=pi_n2b(var);
				while(!(pi_get_DI() & temp))
						usleep(1000);
				sprintf(report,"WC");
			break;
			case PI_WS:
				pi_wait_stop(axis);
				sprintf(report,"WS");
			break;
			case PI_WT:
				// waits for the motion to stop them puts a high
				// on digital out channel # 1 (bit 0?)
				pi_wait_stop(axis);
				pi_DO_up(0x01);	
				sprintf(report,"WT");
			break;

			default:
			sprintf(report,PI_ERROR_REPORT);
			}
		return;
		}
	}
