/* (c) Matt Clark Feb 2003
 *
 * You are free to copy, change and distribute this software BUT
 * you cannot charge for it (even if you change it).  
 *
 * The use of this library in your code conveys no rights over 
 * your code.  So, if you wish to charge for a program that uses 
 * this library that is fine.  You will have to distribute the 
 * library free of charge. 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Matt.Clark@Nottingham.ac.uk
 */
#ifndef __PI_USER__
#define __PI_USER__
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>

#include <pi_cmd.h>
#include <pi_stage.h>
#define VERSION "0.52 pi_user 23 July 2003"


// some useful constants
// how closely to monitor stage when finding an edge
// this is used /10 in the accurate parts...
#define PI_MONITOR_WAIT 10000	

// most -ve and +ve places accepted by the controllers
// source: Navigator Motion Processor Programmers Reference v1.2
#define PI_MOST_POSITIVE	2147483647
#define PI_MOST_NEGATIVE	(-PI_MOST_POSITIVE - 1)

// amount to shift to get past home position.....
// found by experiment / consistent with MP33E, page 9
// this should be user definable - want this feature? request it.
#define PI_FIND_HOME_SHIFT	5000

#define PI_OK		1

// default velocity for moving off limit switches 
// this should be user definable - want this feature? request it.
#define PI_DEF_VEL	50000



#define PI_DEFAULT_DEVICE_NAME "/dev/pi_stage"
// bits in the event status register
#define PI_MOTION_COMPLETE		0x0001
#define PI_WRAP_ARROUND			0x0002
#define PI_BREAKPOINT1			0x0004
#define PI_CAPTURE_RECEIVED		0x0008
#define PI_MOTION_ERROR			0x0010
#define PI_EVENT_POS_LIMIT		0x0020	// note that the chipset defines different
#define PI_EVENT_NEG_LIMIT		0x0040	// bit values for these in different regs..
#define PI_INSTRUCTION_ERR		0x0080
#define PI_COMMUNICATION_ERR		0x0800
#define PI_BREAKPOINT2			0x4000

// bits in the signal status and signal sense registers
#define PI_ENCODER_A			0x0001
#define PI_ENCODER_B			0x0002
#define PI_ENCODER_INDEX		0x0004
#define PI_ENCODER_HOME			0x0008
#define PI_SIGNAL_POS_LIMIT		0x0010	// note that the chipset defines different
#define PI_SIGNAL_NEG_LIMIT		0x0020	// bit values for these in different regs..
#define PI_AXIS_IN			0x0040
#define PI_HALL_A			0x0080
#define PI_HALL_B			0x0100
#define PI_HALL_C			0x0200
#define PI_AXIS_OUT			0x0400
// signal sense register only
#define PI_STEP_OUTPUT			0x0800
#define PI_MOTOR_OUTPUT			0x1000


//bits in the activity status register
#define PI_PHASING_INITD		0x0001	// 1 if phase is init'd (not PI boards)
#define PI_MAX_VELOCITY			0x0002	// 1= max velocity
#define PI_TRACKING			0x0004	// 1= within tracking window (ie pos matches trajectory)
#define PI_PROFILE_MODE_0		0x0008	//	000 = trapeziod 
#define PI_PROFILE_MODE_1		0x0010	//	001 = vel contour
#define PI_PROFILE_MODE_2		0x0020	//	010 = s-curve,	011 = electronic gear
#define PI_PROFILE_MODE			PI_PROFILE_MODE_0|PI_PROFILE_MODE_1|PI_PROFILE_MODE_2
#define PI_RESERVED			0x0040
#define PI_AXIS_SETTLED			0x0080	// 1= axis settled
#define PI_MOTOR_STATUS			0x0100  // 1= motor on
#define PI_POSITION_CAPTURE		0x0200	// 1= pos has been capture, awaiting reading
#define PI_IN_MOTION			0x0400	// 1= still moving
#define PI_ACT_POS_LIMIT		0x0800	// +ve limit is activated (see above event /signal status)
#define PI_ACT_NEG_LIMIT		0x1000	// -ve limit is activated


#define PI_BRAKE_OFF	0
#define PI_BRAKE_ON	1
#define PI_BRAKES_ON	2
#define PI_BRAKES_OFF	3

#define PI_MOTOR_OFF	0
#define PI_MOTOR_ON	1
#define PI_MOTORS_ON	2
#define PI_MOTORS_OFF	3

#define PI_LIMIT_ON	1
#define PI_LIMIT_OFF	0

#define PI_LIMIT_OK		0
#define PI_ON_POSITIVE_LIMIT	1
#define PI_ON_NEGATIVE_LIMIT	2
#define PI_LIMIT_BAD		4

#ifndef BOOL
#define BOOL int
#define TRUE 1
#define FALSE 0
#endif

#define PI_LIMIT_ZERO	0
#define PI_LIMIT_ONE	1
#define PI_LIMIT_FAIL	-1
#define PI_MOVE_OFF_LIMITS 5000
#ifndef BYTE
#define BYTE unsigned char
#endif
#define PI_AXIS	int

// useful bitsnbobs for bit twiddling etc
#define	PI_AXIS_BITS			0x00f0
#define PI_AXIS_SHIFT_BITS		4




// permit debugging
#ifdef PI_DEBUG
#define PDEBUG(a,args...)	printf(a,## args);
#else
#define PDEBUG(a,args...)
#endif
// structure for asyn events
struct pi_stage_event_t{
        int index;
        char axis;
        unsigned long ts_high,ts_low;
        __u32 event;
        };

// IO primatives
inline int	pi_io_c(PI_CMD_ONLY);
inline int	pi_io_cw(PI_CMD_WRITE1, PI_DATA);
inline int	pi_io_cww(PI_CMD_WRITE2, PI_DATA, PI_DATA);
inline int	pi_io_cwww(PI_CMD_WRITE3, PI_DATA, PI_DATA, PI_DATA);
inline int	pi_io_cr(PI_CMD_READ1, PI_DATA *);
inline int	pi_io_crr(PI_CMD_READ2, PI_DATA *);
inline int	pi_io_cwr(PI_CMD_WRITE_READ1, PI_DATA, PI_DATA *);
inline int	pi_io_cwrr(PI_CMD_WRITE_READ2, PI_DATA, PI_DATA *);
void		pi_move_abs(PI_AXIS, int);
void		pi_move_abs(PI_AXIS, int, PI_AXIS, int);
void		pi_move_abs(PI_AXIS, int, PI_AXIS, int, PI_AXIS, int);
void		pi_move_abs(PI_AXIS, int, PI_AXIS, int, PI_AXIS, int, PI_AXIS, int);
void		pi_move_rel(PI_AXIS, int position);
void		pi_move_rel(PI_AXIS, int, PI_AXIS, int);
void		pi_move_rel(PI_AXIS, int, PI_AXIS, int, PI_AXIS, int);
void		pi_move_rel(PI_AXIS, int, PI_AXIS, int, PI_AXIS, int, PI_AXIS, int);
void 		pi_vector_abs(PI_AXIS, int, PI_AXIS, int, int);
void 		pi_vector_rel(PI_AXIS, int, PI_AXIS, int, int);

BOOL		pi_get_refsignal(PI_AXIS);
BOOL 		pi_select_board(BYTE);
int 		pi_getnumberofinstalledboards(void);
int 		pi_axis_installed(void);
BOOL		pi_openboard(void);
BOOL		pi_openboard(char *);
BOOL		pi_closeboard(void);
BOOL		pi_init_board(void);
void		pi_reset_board(void);
void		pi_init_ls(PI_AXIS ,int);
void		pi_set_motor(PI_AXIS ,int);
void		pi_set_brake(PI_AXIS ,int);
BOOL		pi_init_axis(PI_AXIS);
int		pi_auto_set_limit(PI_AXIS);
void		pi_set_limit(PI_AXIS,int);
void 		pi_move_off_limit_switch(PI_AXIS);
int 		pi_find_home(PI_AXIS);
BOOL		pi_moving(PI_AXIS);
int		pi_get_pos(PI_AXIS);
int		pi_get_pos_err(PI_AXIS);
int		pi_getQMC(int, PI_AXIS);
int		pi_getQMC(int, PI_AXIS, int);
int		pi_getQMCA(int, PI_AXIS, int);
void		pi_setQMC(int);
void		pi_setQMC(int, PI_AXIS);
void		pi_setQMC(int, PI_AXIS, int);
void		pi_setQMC(int, PI_AXIS, int, int);
void		pi_setQMCA(int, PI_AXIS, int, int);
int		pi_get_activity_status(PI_AXIS);
int		pi_get_event_status(PI_AXIS);
int		pi_get_signal_status(PI_AXIS);
int		pi_get_v(PI_AXIS);
int             pi_get_vel_cps(PI_AXIS);
void            pi_set_vel_cps(PI_AXIS,int);
void		pi_wait_stop(PI_AXIS);
void		pi_stop_dead(PI_AXIS axis);
void		pi_stop_dead(void);
int		pi_motor_and_brake_status(void);
int		pi_get_motor(PI_AXIS);
int		pi_get_brake(PI_AXIS);
int		pi_read_async_events(struct pi_stage_event_t *, int);
void		pi_enable_async_capture(void (*)(int));
// functions to handle the DIO
void		pi_DO_up(unsigned char bit);
void		pi_DO_down(unsigned char output);
void		pi_DO_set(unsigned char data);
unsigned char	pi_n2b(int n);
int		pi_get_DI();

// additional functions
int		pi_get_limit_status(PI_AXIS);
int		pi_get_limit_status(void);


#endif
